#!/bin/sh
set -u

#ConfPath: auth.conf, engine.conf, persistent.conf, terminal.conf version. Also product.conf if ProductConfPath is not set.
ConfPath="/data/data/com.lightwan.AcceleratorUI"
#ProductConfPath: product.conf
ProductConfPath="/data/data/com.lightwan.AcceleratorUI"
#WorkPath: lightwan, dispatch, lightwan.sh
WorkPath="/data/data/com.lightwan.AcceleratorUI"
#PcapPath: packet capture path, leave it blank for default path
PcapPath="/data/data/com.lightwan.AcceleratorUI"
logName="lightwan-sh.log"
version=""

action="start"
Mode=0
status=""
logPath=""

#Exit Error Code
ER_tun99_exist=1
ER_product_conf_not_exit=2
ER_add_rule_fail=3
ER_lightwan_start_fail=4
ER_probe_lightwan_fail=5
ER_acce_add_rule_fail=6
ER_video_add_rule_fail=7
ER_lightwan_not_start=8
ER_dispatch_not_exit=9
ER_dispatch_set_fail=10
ER_missing_conf_files=11
ER_lightwan_not_exist=12
#ER_lightwan_not_started=255

videoPortRange="5060"

check_ip6_exist()
{
    ip6tables -L > /dev/null 2>&1 && {
        ip6Enable=1
    }
}

get_log_path()
{
    logPathRoot=`cat ${ProductConfPath}/product.conf | grep LogPath`
    if [ -n ${logPathRoot} ]; then
        logPath=${logPathRoot#*=}"/"${logName}
    fi
}

logcat() #info
{
    get_log_path
    if [ -n ${logPath} ]; then
        echo "$(date) $version lightwan_sh $status ${@}" >>${logPath}
    else
        echo "$(date) $version lightwan_sh $status ${@}"
    fi
}

log_exit() #cmd #errno #details
{
    logcat "${1} exit $2 $3"
    echo "${1} exit $2 info: $3"
    if [ ${status} == "[Start]" ];then
        lightwan_stop
    fi
    exit ${2}
}

init_path()
{
    if [ -e $WorkPath ]; then
        if [ -e "${WorkPath}/lightwan" ]; then
            if [ ! -x "${WorkPath}/lightwan" ];then
                chmod 751 ${WorkPath}/lightwan
            fi
        else
            err_msg="Could not find lightwan in $WorkPath"
            log_exit "$0 $1 failed, lightwan not exist" $ER_lightwan_not_exist "${err_msg}"
        fi
        if [ -e "${WorkPath}/dispatch" ]; then
            if [ ! -x "${WorkPath}/dispatch" ];then
                chmod 751 ${WorkPath}/dispatch
            fi
        else
            err_msg="Could not find dispatch in $WorkPath"
            log_exit "$0 $1 failed, dispatch not exist" $ER_dispatch_not_exit "${err_msg}"
        fi
    fi

    if [ ! -f $ConfPath/auth.conf -o ! -f $ConfPath/engine.conf ];then
        err_msg="Could not find auth.conf engine.conf in $ConfPath"
        log_exit "$0 $1 failed, missing conf files" $ER_missing_conf_files "${err_msg}"
    fi

    if [ ! -n "$ProductConfPath" ];then
        ProductConfPath=$ConfPath
    fi

    if [ ! -f ${ProductConfPath}/product.conf ];then
        err_msg="Could not find product.conf in $ProductConfPath"
        log_exit "product.conf does not exist" $ER_product_conf_not_exit "${err_msg}"
    fi
}

check_start_conditions()
{
    tun99_exist=`ifconfig|grep -ci "tun99"`
    if [ $tun99_exist -ne 0 ];then
        err_msg="failed to start lightwan, tun99 is required, but it is occupied. 
        maybe you can run lightwan.sh stop and restart."
        logcat "tun99 occupied" $ER_tun99_exist "${err_msg}"
        echo "tun99 occupied" $ER_tun99_exist "${err_msg}"
        exit $ER_tun99_exist
    fi
}

video_acce_del() #videoPortRange
{
    local video_port_range=${1}    
    local intf=${2}
    
    local check=`iptables -t mangle -nvL FORWARD|grep lightwan_in |grep ${video_port_range}| grep ${intf}`
    while [ -n "$check" ]
    do
        local cmd="iptables -t mangle -D FORWARD -i ${intf} -p tcp --sport ${video_port_range} -m mark ! --mark 999 -j lightwan_in"
        ${cmd} 2>/dev/null
        logcat "${cmd} $?"
        check=`iptables -t mangle -nvL FORWARD|grep lightwan_in |grep ${video_port_range}|grep ${intf}`
    done
    
    check=`iptables -t mangle -nvL FORWARD|grep lightwan_out |grep ${video_port_range}|grep ${intf}`
    while [ -n "$check" ]
    do
        cmd="iptables -t mangle -D FORWARD -o ${intf} -p tcp --dport ${video_port_range} -m mark ! --mark 999 -j lightwan_out"
        ${cmd} 2>/dev/null
        logcat "${cmd} $?"
        check=`iptables -t mangle -nvL OUTPUT|grep lightwan_out |grep ${video_port_range}|grep ${intf}`
    done

    [ $ip6Enable -eq 1 ] && {
        check=`ip6tables -t mangle -nvL FORWARD|grep lightwan_in |grep ${video_port_range}|grep ${intf}`
        while [ -n "$check" ]
        do
            cmd="ip6tables -t mangle -D FORWARD -i ${intf} -p tcp --sport ${video_port_range} -m mark ! --mark 999 -j lightwan_in"
            ${cmd} 2>/dev/null
            logcat "${cmd} $?"
            check=`ip6tables -t mangle -nvL INPUT|grep lightwan_in |grep ${video_port_range}|grep ${intf}`
        done
        
        check=`ip6tables -t mangle -nvL FORWARD|grep lightwan_out |grep ${video_port_range}|grep ${intf}`
        while [ -n "$check" ]
        do
            cmd="ip6tables -t mangle -D FORWARD -i ${intf} -p tcp --sport ${video_port_range} -m mark ! --mark 999 -j lightwan_in"
            ${cmd} 2>/dev/null
            logcat "${cmd} $?"
            check=`ip6tables -t mangle -nvL OUTPUT|grep lightwan_out |grep ${video_port_range}|grep ${intf}`
        done
    }
}

video_acce_safe_add() #videoPortRange
{
    local video_port_range=${1}    
    local intf=${2}

    video_acce_del ${video_port_range} ${intf}
    
    local cmd="iptables -t mangle -A FORWARD -i ${intf} -p tcp --sport ${video_port_range} -m mark ! --mark 999 -j lightwan_in"
    local details="failed to add video-related lightwan_in rules to iptables! script execution interrupted and exited."
    local rule_exist=$(iptables -t mangle -nvL FORWARD | grep "$intf" | grep "$video_port_range" | grep lightwan_in)   
    if [ -z "$rule_exist" ]; then 
        ${cmd} || log_exit "${cmd}" $ER_video_add_rule_fail "${details}"
        logcat ${cmd}
    fi

    cmd="iptables -t mangle -A FORWARD -o ${intf} -p tcp --dport ${video_port_range} -m mark ! --mark 999 -j lightwan_out"
    details="failed to add video-related lightwan_out rules to iptables! script execution interrupted and exited."
    rule_exist=$(iptables -t mangle -nvL FORWARD | grep "$intf" | grep "$video_port_range" | grep lightwan_out)
    if [ -z "$rule_exist" ]; then 
        ${cmd} || log_exit "${cmd}" $ER_video_add_rule_fail "${details}"
        logcat ${cmd}
    fi

    [ $ip6Enable -eq 1 ] && {
        cmd="ip6tables -t mangle -A FORWARD -i ${intf} -p tcp --sport ${video_port_range} -m mark ! --mark 999 -j lightwan_in"
        details="failed to add video-related lightwan_in rules to ip6tables! script execution interrupted and exited."
        rule_exist=$(ip6tables -t mangle -nvL FORWARD | grep "$intf" | grep "$video_port_range" | grep lightwan_in)
        if [ -z "$rule_exist" ]; then 
            ${cmd} || log_exit "${cmd}" $ER_video_add_rule_fail "${details}"
            logcat ${cmd}
        fi

        cmd="ip6tables -t mangle -A FORWARD -o ${intf} -p tcp --dport ${video_port_range} -m mark ! --mark 999 -j lightwan_out"
        details="failed to add video-related lightwan_out rules to ip6tables! script execution interrupted and exited."
        rule_exist=$(ip6tables -t mangle -nvL FORWARD | grep "$intf" | grep "$video_port_range" | grep lightwan_out)
        if [ -z "$rule_exist" ]; then 
            ${cmd} || log_exit "${cmd}" $ER_video_add_rule_fail "${details}"
            logcat ${cmd}
        fi
    }
}

lightwan_start() #videoPortRange #probePortRange #acceif
{
    #stop nss
    /etc/init.d/qca-nss-ecm stop
    #for sip monitor process
    #clean the sip in auth.conf, ready to get the new one
    if [ `grep -c -i "Sip=" "$ConfPath/auth.conf"` -ne '0' ];then
        busybox sed -i '/Sip=/d' "$ConfPath/auth.conf"
    fi

    local sipmonitor=`grep -c -i "IsNeedSipMonitor=TRUE" "$ProductConfPath/product.conf"`
    if [ $sipmonitor -ne '0' ];then
        logcat "start lightwan_sipmonitor process"
        $WorkPath/lightwan -M start & >/dev/null
    fi
    
    ulimit -c unlimited
    echo 1 > /proc/sys/kernel/core_uses_pid
    echo /var/core/core-%e-%p-%t > /proc/sys/kernel/core_pattern
    check_start_conditions

    local video_port_range=${1}
    local start_args=""

    start_args="-p ${ConfPath} -e -m ${Mode} -P $ProductConfPath"
    #run cpe
    logcat "cmd lightwan start ${start_args}"
    $WorkPath/lightwan ${start_args} || {
        lightwan_stop ${videoPortRange}
        log_exit "lightwan start failed" $ER_lightwan_start_fail "lightwan start failed, because something wrong with lightwan file"
    }
    
    logcat "lightwan ${start_args} success"

    sleep 15

    interfaces=$(ip -o addr show | awk '/inet/ {print $2}')
    for interface in $interfaces; do
        intf_name=$(echo $interface | cut -d ":" -f 1)
        echo "intf: $intf_name"
        video_acce_safe_add ${video_port_range} ${intf_name}
    done

    ./dispatch set_log_level/"acs=0,50,30 ipset=0,50,30 netio=0,5,30 datapatch=0,5,30 lwconn=0,5,30 ratelimit=0,5,30 lsw=0,5,30 engine=0,5,50 default=0,5,150"
    logcat "cmd lightwan start completed 0"
    echo "cmd lightwan start completed 0"
}

lightwan_stop()
{
    logcat "cmd lightwan_sipmonitor stop"
    ${WorkPath}/lightwan -M stop >/dev/null
    
    logcat "cmd lightwan stop"
    
    _cmd="iptables -X lightwan_in -t mangle"
    ${_cmd} 2>/dev/null
    logcat "${_cmd} $?"
    _cmd="iptables -X lightwan_out -t mangle"
    ${_cmd} 2>/dev/null
    logcat "${_cmd} $?"

    cmd="ip6tables -X lightwan_in -t mangle"
    ${cmd} 2>/dev/null
    logcat "${cmd} $?"
    _cmd="ip6tables -X lightwan_out -t mangle"
    ${_cmd} 2>/dev/null
    logcat "${_cmd} $?"

    ${WorkPath}/lightwan -D quit 2>/dev/null
    stop_ret=$?
    logcat "${WorkPath}/lightwan quit $stop_ret"
    sleep 3

    #Delete all rules in the FORWARD chain with a target of lightwan_in/lightwan_out
    while iptables -t mangle -nvL FORWARD | grep lightwan_in >/dev/null 2>&1;
    do
        line_number=$(iptables -t mangle -nvL FORWARD --line-numbers | grep lightwan_in | head -n 1 | awk '{print $1}')
        _cmd="iptables -t mangle -D FORWARD $line_number"
        ${_cmd} >/dev/null 2>&1
        logcat "${_cmd} $?"
    done

    while iptables -t mangle -nvL FORWARD | grep lightwan_out >/dev/null 2>&1;
    do
        line_number=$(iptables -t mangle -nvL FORWARD --line-numbers | grep lightwan_out | head -n 1 | awk '{print $1}')
        _cmd="iptables -t mangle -D FORWARD $line_number"
        ${_cmd} >/dev/null 2>&1
        logcat "${_cmd} $?"
    done

    if [ $stop_ret -eq 0 ];then
        echo "cmd lightwan stop completed $stop_ret"
    else
        echo "cmd lightwan stop failed with error code: $stop_ret"
    fi
    exit $stop_ret
}

show_version()
{
    if [ -f ${ConfPath}/version ];then
        version=`cat ${ConfPath}/version`
    else
        version="version file not found"
    fi
    echo $version
}

for opt in ${@};
do
    str="="
    result=$(echo $opt | grep "$str")
    if [ -n $result ]; then
        k=${opt%=*}
        v=${opt#*=}
    else
        k=${opt} 
    fi

    case ${k} in
        "start") action="start";;
        "stop")  action="stop";;
        "--version"|"-v") action="show_lightwan_version";;
        "--mode"|"-m") Mode=${v};;
        "--conf_path"|"-cp") ConfPath=${v};;
        "--product_conf_path"|"-pcp") ProductConfPath=${v};;
        "--work_path"|"-wp") WorkPath=${v};;
        "--pcap_path"|"-pcap") PcapPath=${v};;
        "--video_port_range"|"-apr") videoPortRange=${v};; 
        *) echo "${opt} is error";;
    esac
done

init_path
check_ip6_exist

if [ ${action} = "start" ]; then
    status="[Start]"
    lightwan_start ${videoPortRange}
elif [ ${action} = "stop" ]; then
    status="[Stop]"
    lightwan_stop ${videoPortRange}
elif [ ${action} = "show_lightwan_version" ];then
    status="[Version]"
    show_version
fi